package nl.iobyte.themepark.api.config;

import nl.iobyte.themepark.api.config.enums.StorageKey;
import nl.iobyte.themepark.api.config.enums.StorageLocation;
import nl.iobyte.themepark.api.config.objects.Configuration;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.plugin.Plugin;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ConfigurationManager {

    private final Map<StorageLocation, Configuration> locations = new HashMap<>();

    public ConfigurationManager(Plugin plugin) {
        for(StorageLocation location : StorageLocation.values())
            locations.put(location, new Configuration(plugin, location.getName(), true));
    }

    /**
     * Get configuration instance belonging to a storage location
     * @param location StorageLocation
     * @return Configuration instance
     */
    public Configuration getConfiguration(StorageLocation location) {
        if(location == null)
            return null;

        return locations.get(location);
    }

    /**
     * Save specific storage location
     * @param location StorageLocation
     */
    public void save(StorageLocation location) {
        if(location == null)
            return;

        getConfiguration(location).save();
    }

    /**
     * Save all storage locations
     */
    public void saveAll() {
        for(Configuration configuration : locations.values())
            configuration.save();
    }

    /**
     * Check if configuration contains storage key
     * @param key StorageKey
     * @return Boolean
     */
    public boolean contains(StorageKey key) {
        if(key == null)
            return false;

        return contains(key.getLocation(), key.getPath());
    }

    /**
     * Check if storage location contains path
     * @param location StorageLocation
     * @param path String
     * @return Boolean
     */
    public boolean contains(StorageLocation location, String path) {
        if(location == null)
            return false;

        return getConfiguration(location).contains(path);
    }

    /**
     * Set value to StorageKey
     * @param key StorageKey
     * @param value Object to set StorageKey to
     */
    public void set(StorageKey key, Object value) {
        if(key == null)
            return;

        set(key.getLocation(), key.getPath(), value);
    }

    /**
     * Set value to path in storage location
     * @param location StorageLocation
     * @param path String
     * @param value Object to set path to
     */
    public void set(StorageLocation location, String path, Object value) {
        if(location == null)
            return;

        getConfiguration(location).set(path, value);
    }

    /**
     * Get object value of storage key
     * @param key StorageKey
     * @return Object
     */
    public Object get(StorageKey key) {
        if(key == null)
            return null;

        return get(key.getLocation(), key.getPath());
    }

    /**
     * Get object value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return Object
     */
    public Object get(StorageLocation location, String path) {
        if(location == null)
            return null;

        return getConfiguration(location).get(path);
    }

    /**
     * Get string value of storage key
     * @param key StorageKey
     * @return String
     */
    public String getString(StorageKey key) {
        if(key == null)
            return null;

        return getString(key.getLocation(), key.getPath());
    }

    /**
     * Get string value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return String
     */
    public String getString(StorageLocation location, String path) {
        if(location == null)
            return null;

        return getConfiguration(location).getString(path);
    }

    /**
     * Get int value of storage key
     * @param key StorageKey
     * @return int
     */
    public int getInt(StorageKey key) {
        if(key == null)
            return 0;

        return getInt(key.getLocation(), key.getPath());
    }

    /**
     * Get int value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return int
     */
    public int getInt(StorageLocation location, String path) {
        if(location == null)
            return 0;

        return getConfiguration(location).getInt(path);
    }

    /**
     * Get boolean value of storage key
     * @param key StorageKey
     * @return boolean
     */
    public boolean getBoolean(StorageKey key) {
        if(key == null)
            return false;

        return getBoolean(key.getLocation(), key.getPath());
    }

    /**
     * Get boolean value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return boolean
     */
    public boolean getBoolean(StorageLocation location, String path) {
        if(location == null)
            return false;

        return getConfiguration(location).getBoolean(path);
    }

    /**
     * Get string list value of storage key
     * @param key StorageKey
     * @return List<String>
     */
    public List<String> getStringList(StorageKey key) {
        if(key == null)
            return null;

        return getStringList(key.getLocation(), key.getPath());
    }

    /**
     * Get string list value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return List<String>
     */
    public List<String> getStringList(StorageLocation location, String path) {
        if(location == null)
            return null;

        return getConfiguration(location).getStringList(path);
    }

    /**
     * Get string list value of storage key
     * @param key StorageKey
     * @return List<String>
     */
    public ConfigurationSection getSection(StorageKey key) {
        if(key == null)
            return null;

        return getSection(key.getLocation(), key.getPath());
    }

    /**
     * Get string list value of path in storage location
     * @param location StorageLocation
     * @param path Path
     * @return List<String>
     */
    public ConfigurationSection getSection(StorageLocation location, String path) {
        if(location == null)
            return null;

        return getConfiguration(location).getSection(path);
    }

}
