package nl.iobyte.themepark.api.ridecount.objects;

import nl.iobyte.themepark.ThemePark;
import nl.iobyte.themepark.api.event.ridecount.RideCountAddEvent;
import nl.iobyte.themepark.scheduler.ThemeParkScheduler;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

public class AttractionCount {

    private final String attraction_id;
    private final Map<Integer, Map<Integer, Map<UUID, RideCount>>> counts = new ConcurrentHashMap<>();
    private int current_year, current_month, current_week, current_day;
    private long next_day;

    public AttractionCount(String attraction_id) {
        this.attraction_id = attraction_id;
        updateTimes();
    }

    public String getAttractionID() {
        return attraction_id;
    }

    public Map<Integer, Map<Integer, Map<UUID, RideCount>>> getCounts() {
        return counts;
    }

    public Map<Integer, Map<UUID, RideCount>> getCounts(int year) {
        return counts.get(year);
    }

    public Map<UUID, RideCount> getCounts(int year, int day) {
        Map<Integer, Map<UUID, RideCount>> map = getCounts(year);
        if(map == null)
            return null;

        return map.get(day);
    }

    public void remove() {
        counts.clear();
    }

    public void removeOld(int year, int day) {
        counts.entrySet().removeIf(e -> e.getKey() != year);
        counts.values().forEach(m -> m.entrySet().removeIf(e -> e.getKey() != day));
    }

    private Map<UUID, RideCount> get(int year, int day) {
        Map<Integer, Map<UUID, RideCount>> ym = counts.computeIfAbsent(year, k -> new ConcurrentHashMap<>());
        return ym.computeIfAbsent(day, k -> new ConcurrentHashMap<>());
    }

    public void setCount(UUID uuid, int amount, int total_amount) {
        if(next_day <= System.currentTimeMillis())
            updateTimes();

        Map<UUID, RideCount> counts = get(current_year, current_day);
        counts.computeIfAbsent(uuid, k -> new RideCount(
                uuid,
                attraction_id,
                amount,
                total_amount,
                current_year,
                current_month,
                current_week,
                current_day
        ));
    }

    public RideCount getCount(UUID uuid) {
        if(next_day <= System.currentTimeMillis())
            updateTimes();

        Map<UUID, RideCount> counts = get(current_year, current_day);
        return counts.get(uuid);
    }

    public void addCount(UUID uuid, int amount) {
        if(next_day <= System.currentTimeMillis())
            updateTimes();

        ThemeParkScheduler.runAsync(() -> {
            Map<UUID, RideCount> counts = get(current_year, current_day);
            RideCount rc = counts.computeIfAbsent(uuid, k -> {
                ArrayList<Map<String, Object>> result = ThemePark.getInstance().getAPI().getDatabaseService().executeQuery(
                        "local",
                        "SELECT sum(count) AS total_count FROM counts WHERE uuid=? AND attraction_id=? AND year=? AND month=? GROUP BY uuid",
                        new HashMap<Integer, Object>(){{
                            put(1, uuid);
                            put(2, attraction_id);
                            put(3, current_year);
                            put(4, current_month);
                        }}
                );

                int total_amount = 0;
                if(result != null)
                    for(Map<String, Object> row : result)
                        total_amount = (int) row.get("total_count");

                return new RideCount(
                        uuid,
                        attraction_id,
                        0,
                        total_amount,
                        current_year,
                        current_month,
                        current_week,
                        current_day
                );
            });

            //Event Parameters
            int old = rc.getCount();
            int current = old + amount;

            //Add amount -> execute event
            rc.addCount(amount);
            ThemePark.getInstance().getAPI().getEventDispatcher().call(new RideCountAddEvent(
                    rc,
                    old,
                    current
            ));
        });
    }

    public void removeCount(UUID uuid) {
        for(Map<Integer, Map<UUID, RideCount>> m1 : counts.values())
            for(Map<UUID, RideCount> m2 : m1.values())
                m2.entrySet().removeIf(e -> e.getKey() == uuid && e.getValue().getUpdatedAt() == -1);
    }

    private void updateTimes() {
        Calendar calendar = new GregorianCalendar();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        calendar.set(Calendar.MILLISECOND, 0);

        current_year = calendar.get(Calendar.YEAR);
        current_month = calendar.get(Calendar.MONTH) + 1;
        current_week = calendar.get(Calendar.WEEK_OF_YEAR);
        current_day = calendar.get(Calendar.DAY_OF_YEAR);
        next_day = calendar.getTimeInMillis();
    }

}
